@@ -4,8 +4,10 @@ use super::session::SharedSession;
 use crate::{
     auth::UserDetail,
     server::controlchan::Reply,
+    server::session::TraceId,
     storage::{Error, StorageBackend},
 };
+use std::fmt;
 use tokio::sync::mpsc::{Receiver, Sender};
 
 // Commands that can be send to the data channel / data loop.
@@ -37,6 +39,18 @@ pub enum DataChanCmd {
     },
 }
 
+impl DataChanCmd {
+    /// Returns the path the command pertains to
+    pub fn path(&self) -> Option<String> {
+        match self {
+            DataChanCmd::Retr { path, .. } => Some(path.clone()),
+            DataChanCmd::Stor { path, .. } => Some(path.clone()),
+            DataChanCmd::List { path, .. } => path.clone(),
+            DataChanCmd::Nlst { path, .. } => path.clone(),
+        }
+    }
+}
+
 /// Messages that can be sent to the control channel loop.
 #[derive(Debug)]
 #[allow(dead_code)]
@@ -45,13 +59,17 @@ pub enum ControlChanMsg {
     PermissionDenied,
     /// File not found
     NotFound,
-    /// Send the data to the client
-    SendData {
+    /// Data was successfully sent to the client during a GET
+    SentData {
+        /// The path as specified by the client
+        path: String,
         /// The number of bytes transferred
         bytes: u64,
     },
     /// We've written the data from the client to the StorageBackend
     WrittenData {
+        /// The path as specified by the client
+        path: String,
         /// The number of bytes transferred
         bytes: u64,
     },
@@ -61,8 +79,6 @@ pub enum ControlChanMsg {
     DataConnectionClosedAfterStor,
     /// Failed to write data to disk
     WriteFailed,
-    /// Unknown Error retrieving file
-    UnknownRetrieveError,
     /// Listed the directory successfully
     DirectorySuccessfullyListed,
     /// Failed to list the directory contents
@@ -70,29 +86,50 @@ pub enum ControlChanMsg {
     /// Successfully cwd
     CwdSuccess,
     /// File successfully deleted
-    DelSuccess,
+    DelFileSuccess {
+        /// The path as specified by the client
+        path: String,
+    },
+    /// File successfully deleted
+    RmDirSuccess {
+        /// The path as specified by the client
+        path: String,
+    },
+    /// File successfully deleted
+    RenameSuccess {
+        /// The old path as specified by the client
+        old_path: String,
+        /// The new path as specified by the client
+        new_path: String,
+    },
     /// Failed to delete file
     DelFail,
     /// Quit the client connection
-    Quit,
+    ExitControlLoop,
     /// Successfully created directory
-    MkdirSuccess(std::path::PathBuf),
+    MkDirSuccess { path: String },
     /// Failed to crate directory
     MkdirFail,
     /// Authentication successful
-    AuthSuccess,
+    AuthSuccess { username: String, trace_id: TraceId },
     /// Authentication failed
     AuthFailed,
     /// Sent to switch the control channel to TLS/SSL mode.
     SecureControlChannel,
     /// Sent to switch the control channel from TLS/SSL mode back to plaintext.
     PlaintextControlChannel,
-    /// Errors comming from the storage
+    /// Errors coming from the storage backend
     StorageError(Error),
     /// Reply on the command channel
     CommandChannelReply(Reply),
 }
 
+impl fmt::Display for ControlChanMsg {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        fmt::Debug::fmt(self, f)
+    }
+}
+
 // ProxyLoopMsg is sent to the proxy loop when proxy protocol mode is enabled. See the
 // Server::proxy_protocol_mode and Server::listen_proxy_protocol_mode methods.
 #[derive(Debug)]
