@@ -1,6 +1,9 @@
 use crate::{
     auth::{Authenticator, UserDetail},
     metrics::MetricsMiddleware,
+    notification::DataListener,
+    notification::PresenceListener,
+    server::shutdown,
     server::{
         chancomms::{ControlChanMsg, ProxyLoopSender},
         controlchan::{
@@ -13,8 +16,10 @@ use crate::{
             handler::{CommandContext, CommandHandler},
             log::LoggingMiddleware,
             middleware::ControlChanMiddleware,
+            notify::EventDispatcherMiddleware,
             Reply, ReplyCode,
         },
+        failed_logins::FailedLoginsCache,
         ftpserver::options::{FtpsRequired, PassiveHost, SiteMd5},
         session::SharedSession,
         tls::FtpsConfig,
@@ -23,7 +28,6 @@ use crate::{
     storage::{ErrorKind, Metadata, StorageBackend},
 };
 
-use crate::server::shutdown;
 use async_trait::async_trait;
 use futures_util::{SinkExt, StreamExt};
 use rustls::ServerConnection;
@@ -60,6 +64,8 @@ where
     pub ftps_required_control_chan: FtpsRequired,
     pub ftps_required_data_chan: FtpsRequired,
     pub site_md5: SiteMd5,
+    pub data_listener: Arc<dyn DataListener>,
+    pub presence_listener: Arc<dyn PresenceListener>,
 }
 
 /// Does TCP processing when an FTP client connects
@@ -70,6 +76,7 @@ pub async fn spawn<Storage, User>(
     destination: Option<SocketAddr>,
     proxyloop_msg_tx: Option<ProxyLoopSender<Storage, User>>,
     mut shutdown: shutdown::Listener,
+    failed_logins: Option<Arc<FailedLoginsCache>>,
 ) -> Result<(), ControlChanError>
 where
     User: UserDetail + 'static,
@@ -88,6 +95,8 @@ where
         idle_session_timeout,
         logger,
         site_md5: sitemd5,
+        data_listener,
+        presence_listener,
         ..
     } = config;
 
@@ -98,9 +107,10 @@ where
         .ftps(ftps_config.clone())
         .metrics(collect_metrics)
         .control_msg_tx(control_msg_tx.clone())
-        .destination(destination);
+        .destination(destination)
+        .failed_logins(failed_logins);
 
-    let logger = logger.new(slog::o!("trace-id" => format!("{}", session.trace_id), "source" => format!("{}", session.source)));
+    let mut logger = logger.new(slog::o!("trace-id" => format!("{}", session.trace_id), "source" => format!("{}", session.source)));
 
     let shared_session: SharedSession<Storage, User> = Arc::new(Mutex::new(session));
     let local_addr = tcp_stream.local_addr()?;
@@ -119,6 +129,8 @@ where
         sitemd5,
     };
 
+    let event_chain = EventDispatcherMiddleware::new(data_listener, presence_listener, event_chain);
+
     let event_chain = AuthMiddleware {
         session: shared_session.clone(),
         next: event_chain,
@@ -163,8 +175,14 @@ where
                 let mut incoming = None;
                 let mut timeout_delay = Box::pin(tokio::time::sleep(idle_session_timeout));
                 tokio::select! {
-                    Some(cmd_result) = command_source.next() => {
-                        incoming = Some(cmd_result.map(Event::Command));
+                    cmd = command_source.next() => {
+                        match cmd {
+                            Some(cmd_result) => incoming = Some(cmd_result.map(Event::Command)),
+                            None => {
+                                slog::info!(logger, "Command stream ended.");
+                                incoming = Some(Ok(Event::InternalMsg(ControlChanMsg::ExitControlLoop)))
+                            }
+                        }
                     },
                     Some(msg) = control_msg_rx.recv() => {
                         incoming = Some(Ok(Event::InternalMsg(msg)));
@@ -178,7 +196,7 @@ where
                     },
                     _ = shutdown.listen() => {
                         slog::info!(logger, "Shutting down control loop");
-                        incoming = Some(Ok(Event::InternalMsg(ControlChanMsg::Quit)))
+                        incoming = Some(Ok(Event::InternalMsg(ControlChanMsg::ExitControlLoop)))
                         // TODO: Do we want to wait a bit for a data transfer to complete i.e. session.data_busy is true?
                     }
                 };
@@ -186,7 +204,8 @@ where
             };
             match incoming {
                 None => {} // Loop again
-                Some(Ok(Event::InternalMsg(ControlChanMsg::Quit))) => {
+                Some(Ok(Event::InternalMsg(ControlChanMsg::ExitControlLoop))) => {
+                    let _ = event_chain.handle(Event::InternalMsg(ControlChanMsg::ExitControlLoop)).await;
                     slog::info!(logger, "Exiting control loop");
                     return;
                 }
@@ -227,20 +246,21 @@ where
                         command_source = src;
                     }
 
+                    if let Event::Command(Command::User { username }) = &event {
+                        let s: String = String::from_utf8_lossy(username).into();
+                        logger = logger.new(slog::o!("username" => s));
+                    }
+
                     // TODO: Handle Event::InternalMsg(InternalMsg::PlaintextControlChannel)
 
-                    match event_chain.handle(event).await {
-                        Err(e) => {
-                            slog::warn!(logger, "Event handler chain error: {:?}. Closing control connection", e);
-                            return;
-                        }
-                        Ok(reply) => {
-                            let result = reply_sink.send(reply).await;
-                            if result.is_err() {
-                                slog::warn!(logger, "Could not send reply to client");
-                                return;
-                            }
-                        }
+                    let handle_result = match event_chain.handle(event).await {
+                        Err(e) => Err(e),
+                        Ok(reply) => reply_sink.send(reply).await,
+                    };
+
+                    if let Err(chan_err) = handle_result {
+                        slog::warn!(logger, "Event handler chain error: {:?}. Closing control connection", chan_err);
+                        return;
                     }
                 }
                 Some(Err(e)) => {
@@ -315,7 +335,7 @@ where
         match msg {
             NotFound => Ok(Reply::new(ReplyCode::FileError, "File not found")),
             PermissionDenied => Ok(Reply::new(ReplyCode::FileError, "Permision denied")),
-            SendData { .. } => {
+            SentData { .. } => {
                 let mut session = self.session.lock().await;
                 session.start_pos = 0;
                 Ok(Reply::new(ReplyCode::ClosingDataConnection, "Successfully sent"))
@@ -328,15 +348,12 @@ where
                 Ok(Reply::new(ReplyCode::ClosingDataConnection, "File successfully written"))
             }
             DataConnectionClosedAfterStor => Ok(Reply::new(ReplyCode::FileActionOkay, "unFTP holds your data for you")),
-            UnknownRetrieveError => Ok(Reply::new(ReplyCode::TransientFileError, "Unknown Error")),
             DirectorySuccessfullyListed => Ok(Reply::new(ReplyCode::ClosingDataConnection, "Listed the directory")),
             DirectoryListFailure => Ok(Reply::new(ReplyCode::ClosingDataConnection, "Failed to list the directory")),
-            CwdSuccess => Ok(Reply::new(ReplyCode::FileActionOkay, "Successfully cwd")),
-            DelSuccess => Ok(Reply::new(ReplyCode::FileActionOkay, "File successfully removed")),
+            CwdSuccess => Ok(Reply::new(ReplyCode::FileActionOkay, "Successfully changed working directory")),
+            DelFileSuccess { .. } | RmDirSuccess { .. } => Ok(Reply::new(ReplyCode::FileActionOkay, "Successfully removed")),
             DelFail => Ok(Reply::new(ReplyCode::TransientFileError, "Failed to delete the file")),
-            // The InternalMsg::Quit will never be reached, because we catch it in the task before
-            // this closure is called (because we have to close the connection).
-            Quit => Ok(Reply::new(ReplyCode::ClosingControlConnection, "Bye!")),
+            ExitControlLoop => Ok(Reply::none()),
             SecureControlChannel => {
                 let mut session = self.session.lock().await;
                 session.cmd_tls = true;
@@ -347,9 +364,10 @@ where
                 session.cmd_tls = false;
                 Ok(Reply::none())
             }
-            MkdirSuccess(path) => Ok(Reply::new_with_string(ReplyCode::DirCreated, path.to_string_lossy().to_string())),
+            MkDirSuccess { path } => Ok(Reply::new_with_string(ReplyCode::DirCreated, path)),
             MkdirFail => Ok(Reply::new(ReplyCode::FileError, "Failed to create directory")),
-            AuthSuccess => {
+            RenameSuccess { .. } => Ok(Reply::new(ReplyCode::FileActionOkay, "Renamed")),
+            AuthSuccess { .. } => {
                 let mut session = self.session.lock().await;
                 session.state = WaitCmd;
                 Ok(Reply::new(ReplyCode::UserLoggedIn, "User logged in, proceed"))
