@@ -6,21 +6,22 @@ pub mod options;
 
 use super::{
     controlchan,
+    failed_logins::FailedLoginsCache,
     ftpserver::{error::ServerError, error::ShutdownError, options::FtpsRequired, options::SiteMd5},
     shutdown,
     tls::FtpsConfig,
 };
 use crate::{
     auth::{anonymous::AnonymousAuthenticator, Authenticator, UserDetail},
-    options::{FtpsClientAuth, TlsFlags},
+    notification::{nop::NopListener, DataListener, PresenceListener},
+    options::{FailedLoginsPolicy, FtpsClientAuth, TlsFlags},
+    server::shutdown::Notifier,
     server::{
         proxy_protocol::{ProxyMode, ProxyProtocolSwitchboard},
         tls,
     },
     storage::{Metadata, StorageBackend},
 };
-
-use crate::server::shutdown::Notifier;
 use options::{PassiveHost, DEFAULT_GREETING, DEFAULT_IDLE_SESSION_TIMEOUT_SECS};
 use slog::*;
 use std::{fmt::Debug, future::Future, net::SocketAddr, ops::Range, path::PathBuf, pin::Pin, sync::Arc, time::Duration};
@@ -55,6 +56,8 @@ where
     storage: Arc<dyn (Fn() -> Storage) + Send + Sync>,
     greeting: &'static str,
     authenticator: Arc<dyn Authenticator<User>>,
+    data_listener: Arc<dyn DataListener>,
+    presence_listener: Arc<dyn PresenceListener>,
     passive_ports: Range<u16>,
     passive_host: PassiveHost,
     collect_metrics: bool,
@@ -69,6 +72,7 @@ where
     logger: slog::Logger,
     site_md5: SiteMd5,
     shutdown: Pin<Box<dyn Future<Output = options::Shutdown> + Send + Sync>>,
+    failed_logins_policy: Option<FailedLoginsPolicy>,
 }
 
 impl<Storage, User> Server<Storage, User>
@@ -99,6 +103,8 @@ where
             storage: Arc::from(sbe_generator),
             greeting: DEFAULT_GREETING,
             authenticator,
+            data_listener: Arc::new(NopListener {}),
+            presence_listener: Arc::new(NopListener {}),
             passive_ports: options::DEFAULT_PASSIVE_PORTS,
             passive_host: options::DEFAULT_PASSIVE_HOST,
             ftps_mode: FtpsConfig::Off,
@@ -113,6 +119,7 @@ where
             ftps_trust_store: options::DEFAULT_FTPS_TRUST_STORE.into(),
             site_md5: SiteMd5::default(),
             shutdown: Box::pin(futures_util::future::pending()),
+            failed_logins_policy: None,
         }
     }
 
@@ -300,6 +307,20 @@ where
         self
     }
 
+    /// Sets an [`DataListener`](crate::notification::DataListener) that will
+    /// be notified of data changes that happen in a user's session.
+    pub fn notify_data(mut self, listener: impl DataListener + 'static) -> Self {
+        self.data_listener = Arc::new(listener);
+        self
+    }
+
+    /// Sets an [`PresenceListener`](crate::notification::PresenceListener) that will
+    /// be notified of user logins and logouts
+    pub fn notify_presence(mut self, listener: impl PresenceListener + 'static) -> Self {
+        self.presence_listener = Arc::new(listener);
+        self
+    }
+
     /// Specifies how the IP address that libunftp will advertise in response to the PASV command is
     /// determined.
     ///
@@ -449,6 +470,49 @@ where
         self
     }
 
+    /// Enables a password guessing protection policy
+    ///
+    /// Policy used to temporarily block an account, source IP or the
+    /// combination of both, after a certain number of failed login
+    /// attempts for a certain time.
+    ///
+    /// There are different policies to choose from. Such as to lock
+    /// based on the combination of source IP + username or only
+    /// username or IP. For example, if you choose IP based blocking,
+    /// multiple successive failed login attempts will block any login
+    /// attempt from that IP for a defined period, including login
+    /// attempts for other users.
+    ///
+    /// The default policy is to block on the combination of source IP
+    /// and username. This policy affects only this specific
+    /// IP+username combination, and does not block the user logging
+    /// in from elsewhere.
+    ///
+    /// It is also possible to override the default 'Penalty', which
+    /// defines how many failed login attempts before applying the
+    /// policy, and after what time the block expires.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// use libunftp::Server;
+    /// use libunftp::options::{FailedLoginsPolicy,FailedLoginsBlock};
+    /// use unftp_sbe_fs::ServerExt;
+    ///
+    /// // With default policy
+    /// let server = Server::with_fs("/tmp").failed_logins_policy(FailedLoginsPolicy::default());
+    ///
+    /// // Or choose a specific policy like based on source IP and
+    /// // longer block (maximum 3 attempts, 5 minutes, IP based
+    /// // blocking)
+    /// use std::time::Duration;
+    /// let server = Server::with_fs("/tmp").failed_logins_policy(FailedLoginsPolicy::new(3, Duration::from_secs(300), FailedLoginsBlock::IP));
+    /// ```
+    pub fn failed_logins_policy(mut self, policy: FailedLoginsPolicy) -> Self {
+        self.failed_logins_policy = Some(policy);
+        self
+    }
+
     /// Runs the main FTP process asynchronously. Should be started in a async runtime context.
     ///
     /// # Example
@@ -479,6 +543,8 @@ where
         let bind_address: SocketAddr = bind_address.into().parse()?;
         let shutdown_notifier = Arc::new(shutdown::Notifier::new());
 
+        let failed_logins = self.failed_logins_policy.as_ref().map(|policy| FailedLoginsCache::new(policy.clone()));
+
         let listen_future = match self.proxy_protocol_mode {
             ProxyMode::On { external_control_port } => Box::pin(
                 listen_proxied::ProxyProtocolListener {
@@ -488,6 +554,7 @@ where
                     options: (&self).into(),
                     proxy_protocol_switchboard: Some(ProxyProtocolSwitchboard::new(self.logger.clone(), self.passive_ports.clone())),
                     shutdown_topic: shutdown_notifier.clone(),
+                    failed_logins: failed_logins.clone(),
                 }
                 .listen(),
             ) as Pin<Box<dyn Future<Output = std::result::Result<(), ServerError>> + Send>>,
@@ -497,13 +564,22 @@ where
                     logger: self.logger.clone(),
                     options: (&self).into(),
                     shutdown_topic: shutdown_notifier.clone(),
+                    failed_logins: failed_logins.clone(),
                 }
                 .listen(),
             ) as Pin<Box<dyn Future<Output = std::result::Result<(), ServerError>> + Send>>,
         };
 
+        let sweeper_fut = if let Some(ref failed_logins) = failed_logins {
+            Box::pin(failed_logins.sweeper(self.logger.clone(), shutdown_notifier.clone())) as Pin<Box<dyn futures_util::Future<Output = ()> + Send>>
+        } else {
+            Box::pin(futures_util::future::pending()) as Pin<Box<dyn futures_util::Future<Output = ()> + Send>>
+        };
         tokio::select! {
             result = listen_future => result,
+            _ = sweeper_fut => {
+                Ok(())
+            },
             opts = self.shutdown => {
                 slog::debug!(logger, "Shutting down within {:?}", opts.grace_period);
                 shutdown_notifier.notify().await;
@@ -548,6 +624,8 @@ where
             ftps_required_control_chan: server.ftps_required_control_chan,
             ftps_required_data_chan: server.ftps_required_data_chan,
             site_md5: server.site_md5,
+            data_listener: server.data_listener.clone(),
+            presence_listener: server.presence_listener.clone(),
         }
     }
 }
@@ -574,6 +652,7 @@ where
             .field("ftps_trust_store", &self.ftps_trust_store)
             .field("idle_session_timeout", &self.idle_session_timeout)
             .field("proxy_protocol_mode", &self.proxy_protocol_mode)
+            .field("failed_logins_policy", &self.failed_logins_policy)
             .finish()
     }
 }
